/******************************************************************
*
*	CyberUPnP for Java
*
*	Copyright (C) Satoshi Konno 2002-2003
*
*	File : SampleClock.java
*
******************************************************************/

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import org.cybergarage.util.*;

import org.cybergarage.upnp.device.*;

public class WasherFrame extends JFrame implements Runnable, WindowListener 
{
	private final static String TITLE = "CyberLink Sample Clock";
	private WasherDevice clockDev;
	private WasherPane clockPane;
	
	public WasherFrame()
	{
		super(TITLE);

		try {
			clockDev = new WasherDevice();
		}
		catch (InvalidDescriptionException e) {
			Debug.warning(e);
		}
				
		getContentPane().setLayout(new BorderLayout());

		clockPane = new WasherPane();		
		getContentPane().add(clockPane, BorderLayout.CENTER);

		addWindowListener(this);
		
		pack();
		setVisible(true);
	}

	public WasherPane getClockPanel()
	{
		return clockPane;
	}

	public WasherDevice getClockDevice()
	{
		return clockDev;
	}
		
	////////////////////////////////////////////////
	//	run	
	////////////////////////////////////////////////

	private Thread timerThread = null;
		
	public void run()
	{
		Thread thisThread = Thread.currentThread();

		while (timerThread == thisThread) {
			getClockDevice().update();
			getClockPanel().repaint();
			try {
				Thread.sleep(1000);
			}
			catch(InterruptedException e) {}
		}
	}
	
	public void start()
	{
		clockDev.start();
		
		timerThread = new Thread(this);
		timerThread.start();
	}
	
	public void stop()
	{
		clockDev.stop();
		timerThread = null;
	}

	////////////////////////////////////////////////
	//	main
	////////////////////////////////////////////////
	
	public void windowActivated(WindowEvent e) 
	{
	}
	
	public void windowClosed(WindowEvent e) 
	{
	}
	
	public void windowClosing(WindowEvent e) 
	{
		stop();
		System.exit(0);
	}
	
	public void windowDeactivated(WindowEvent e) 
	{
	}
	
	public void windowDeiconified(WindowEvent e) 
	{
	}
	
	public void windowIconified(WindowEvent e) 
	{
	}
	
	public void windowOpened(WindowEvent e)
	{
	}
	

	////////////////////////////////////////////////
	//	main
	////////////////////////////////////////////////

	public static void main(String args[]) 
	{
		//Debug.on();

		WasherFrame sampClock = new WasherFrame();
		sampClock.start();
		
		WasherDevice clockDev = sampClock.getClockDevice();
	}

}

