/******************************************************************
*
*	CyberUPnP for Java
*
*	Copyright (C) Satoshi Konno 2002
*
*	File : ClockDevice.java
*
******************************************************************/

import java.io.*;

import org.cybergarage.upnp.*;
import org.cybergarage.upnp.device.*;
import org.cybergarage.upnp.control.*;

public class ClockDevice extends Device implements ActionListener, QueryListener
{
	private final static String DESCRIPTION_FILE_NAME = "description/description.xml";

	private StateVariable timeVar;
	
	public ClockDevice() throws InvalidDescriptionException
	{
		super(new File(DESCRIPTION_FILE_NAME));

		Action getTimeAction = getAction("GetTime");
		getTimeAction.setActionListener(this);
		
		Action setTimeAction = getAction("SetTime");
		setTimeAction.setActionListener(this);
		
		ServiceList serviceList = getServiceList();
		Service service = serviceList.getService(0);
		service.setQueryListener(this);

		timeVar = getStateVariable("Time");
	}

	////////////////////////////////////////////////
	// ActionListener
	////////////////////////////////////////////////

	public boolean actionControlReceived(Action action)
	{
		String actionName = action.getName();
		if (actionName.equals("GetTime") == true) {
			Clock clock = Clock.getInstance();
			String dateStr = clock.getDateString();
			Argument timeArg = action.getArgument("CurrentTime");
			timeArg.setValue(dateStr);
			return true;
		}
		if (actionName.equals("SetTime") == true) {
			Argument timeArg = action.getArgument("NewTime");
			String newTime = timeArg.getValue();
			Argument resultArg = action.getArgument("Result");
			resultArg.setValue("Not implemented (" + newTime + ")");
			return true;
		}
		return false;
	}

	////////////////////////////////////////////////
	// QueryListener
	////////////////////////////////////////////////

	public String queryControlReceived(Service service, String varName)
	{
		Clock clock = Clock.getInstance();
		return clock.getDateString();
	}

	////////////////////////////////////////////////
	// update
	////////////////////////////////////////////////

	public void update()
	{
			Clock clock = Clock.getInstance();
			String timeStr = clock.toString();
			timeVar.setValue(timeStr);
	}			
}

