/******************************************************************
*
*	CyberUPnP for Java
*
*	Copyright (C) Satoshi Konno 2002
*
*	File : SampleClockPane.java
*
******************************************************************/

import java.io.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;

import javax.swing.*;
import javax.imageio.ImageIO;

import org.cybergarage.util.*;

public class TvPane extends JPanel  // MouseListener
{
	private final static int IMAGE_BORDER_SIZE = 20;
	private final static int IMAGE_BOTTOM_BORDER_SIZE = 30;
	
	////////////////////////////////////////////////
	//	Constructor
	////////////////////////////////////////////////

	public TvPane()
	{
		loadImage();
		initPanel();
	}

	////////////////////////////////////////////////
	//	TvDevice
	////////////////////////////////////////////////

	private TvDevice tvDev = null;
	
	public void setDevice(TvDevice dev)
	{
		tvDev = dev;
	}

	public TvDevice getDevice()
	{
		return tvDev;
	}
	
	////////////////////////////////////////////////
	//	Background
	////////////////////////////////////////////////

	private final static String CLOCK_PANEL_IMAGE = "images/tv.jpg";

	private BufferedImage panelmage;
	private BufferedImage panelmage;
	
	private void loadImage()
	{
		File f = new File(CLOCK_PANEL_IMAGE);
		try {
			panelmage = ImageIO.read(f);
		}
		catch (Exception e) {
			Debug.warning(e);
		}
	}

	private BufferedImage getPaneImage()
	{
		return panelmage;
	}

	////////////////////////////////////////////////
	//	Background
	////////////////////////////////////////////////

	private void initPanel()
	{
		BufferedImage panelmage = getPaneImage();
		setPreferredSize(new Dimension(panelmage.getWidth(), panelmage.getHeight()));
	}

	////////////////////////////////////////////////
	//	Font
	////////////////////////////////////////////////

	private final static String DEFAULT_FONT_NAME = "Lucida Console";
	private final static int DEFAULT_TIME_FONT_SIZE = 10;

	private Font timeFont = null;

	private Font getFont(Graphics g, int size)
	{
		Font font = new Font(DEFAULT_FONT_NAME, Font.PLAIN, size);
		if (font != null)
			return font;
		return g.getFont();
	}
		
	private Font getTimeFont(Graphics g)
	{
		if (timeFont == null)
			timeFont = getFont(g, DEFAULT_TIME_FONT_SIZE);
		return timeFont;
	}

	////////////////////////////////////////////////
	//	paint
	////////////////////////////////////////////////

	private void drawClockInfo(Graphics g)
	{
		TvDevice tvDev = getDevice();

		int winWidth = getWidth();
		int winHeight = getHeight();
		
		g.setColor(Color.WHITE);
		
		//// Time String ////
		
		String timeStr = tvDev.getClockTime();

		Font timeFont = getTimeFont(g);
		g.setFont(timeFont);

		FontMetrics timeFontMetric = g.getFontMetrics();
		Rectangle2D timeStrBounds = timeFontMetric.getStringBounds(timeStr, g);

		int timeStrWidth = (int)timeStrBounds.getWidth();		
		int timeStrHeight = (int)timeStrBounds.getHeight();
		int timeStrX = IMAGE_BORDER_SIZE;
		int timeStrY = IMAGE_BORDER_SIZE + timeStrHeight;
		g.drawString(
			timeStr,
			timeStrX,
			timeStrY);
	}

	private void drawTvImage(Graphics g)
	{
		g.setColor(Color.WHITE);
		g.clearRect(IMAGE_BORDER_SIZE, IMAGE_BORDER_SIZE, getWidth()-IMAGE_BORDER_SIZE*2, getHeight()-IMAGE_BORDER_SIZE-IMAGE_BOTTOM_BORDER_SIZE);
	}
	
	private void clear(Graphics g)
	{
		g.setColor(Color.GRAY);
		g.clearRect(0, 0, getWidth(), getHeight());
	}
	

	private void drawPanelImage(Graphics g)
	{
		g.drawImage(getPaneImage(), 0, 0, null);
	}
		
	public void paint(Graphics g)
	{
		clear(g);
		drawPanelImage(g);
		TvDevice tvDev = getDevice();
		if (tvDev.isOn() == true) {
			drawTvImage(g);
			drawClockInfo(g);
		}
	}
}

