/******************************************************************
*
*	CyberUPnP for Java
*
*	Copyright (C) Satoshi Konno 2002
*
*	File: ControlRequest.java
*
*	Revision;
*
*	01/29/03
*		- first revision.
*	05/22/03
*		- Giordano Sassaroli <sassarol@cefriel.it>
*		- Description: inserted a check at the beginning of the setRequestHost method
*		- Problem : If the host does not start with a '/', the device could refuse the control action
*		- Error : it is not an error, but adding the '/' when missing allows the integration with the Intel devices
*
******************************************************************/

package org.cybergarage.upnp.control;

import org.cybergarage.http.*;
import org.cybergarage.soap.*;

import org.cybergarage.upnp.*;

public class ControlRequest extends SOAPRequest
{
	////////////////////////////////////////////////
	//	Constructor
	////////////////////////////////////////////////
	
	public ControlRequest()
	{
	}

	public ControlRequest(HTTPRequest httpReq)
	{
		set(httpReq);
	}

	////////////////////////////////////////////////
	//	Query
	////////////////////////////////////////////////

	public boolean isQueryControl()
	{
		return isSOAPAction(Control.QUERY_SOAPACTION);
	}
	
	public boolean isActionControl()
	{
		return !isQueryControl();
	}

	////////////////////////////////////////////////
	//	setRequest
	////////////////////////////////////////////////

	protected void setRequestHost(Service service)
	{
		String ctrlURL = service.getControlURL();

		// Thanks for Giordano Sassaroli <sassarol@cefriel.it> (05/21/03)
		setURI(ctrlURL, true);

		Device rootDev = service.getRootDevice();

		String urlBaseStr = rootDev.getURLBase();
		String reqHost = HTTP.getHost(urlBaseStr);
		int reqPort = HTTP.getPort(urlBaseStr);
		String requestHost = ctrlURL;
		if (HTTP.isAbsoluteURL(ctrlURL) == false)
			requestHost = HTTP.getURL(urlBaseStr, ctrlURL);
		setHost(reqHost, reqPort);
		setRequestHost(reqHost);
		setRequestPort(reqPort);
	}
	
}

